/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1998-1999 Jeremie Miller.
 *
 * Portions Copyright (c) 2002 Matthias Wimmer.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 * --------------------------------------------------------------------------*/
 
#include "smtp.h"

/*

This is the smtp server component.

It acts as a simple SMTP server for both receiving email directly from another server or for delivering
email to another server.

What inspired me to write this very frightening thing is a mystery :) 

Jer

*/

/* takes foo@mapped.host or foo%jabber.org@smtp.host and translates to foo@jabber.org */
jid smtp_mx2(smtpi i, jid id)
{
    char *at, *trhost;
    xmlnode tr;

    if(id == NULL || id->user == NULL) return NULL;

    log_debug(ZONE,"mx2 checking %s",jid_full(id));

    /* query for any translation */
    tr = xmlnode_get_tag(i->config, spools(id->p, "map?mxhost=",id->server,id->p));
    trhost = xmlnode_get_attrib(tr,"jabber");

    if(trhost != NULL)
    {
        jid_set(id,trhost,JID_SERVER);
    }else{
        if((at = strstr(id->user,"%")) != NULL)
        {
            *at = '@';
            id = jid_new(id->p,id->user);

            /* check to make sure this id is valid to relay between */
            if(id != NULL && i->open == 0 && xmlnode_get_tag(i->config,spools(id->p,"open=",id->server,id->p)) == NULL)
                return NULL;
        }else{
            return NULL;
        }
    }

    log_debug(ZONE,"mx2 returning %s",jid_full(id));
    return id;
}

/* takes foo@jabber.org and translates to foo@mapped.host or foo%jabber.org@smtp.host */
jid smtp_2mx(smtpi i, jid from)
{
    char *at, *trhost, *strid;
    jid id;
    xmlnode tr;

    if(from == NULL || from->user == NULL) return NULL;

    log_debug(ZONE,"2mx checking %s",jid_full(from));

    /* copy jid and modify copy for the result */
    id = jid_new(from->p,jid_full(from));
    jid_set(id,NULL,JID_RESOURCE);

    /* query for any translation */
    tr = xmlnode_get_tag(i->config, spools(id->p, "map?jabber=",id->server,id->p));
    trhost = xmlnode_get_attrib(tr,"mxhost");

    if(trhost != NULL)
    {
        jid_set(id,trhost,JID_SERVER);
    }else{
        strid = jid_full(id);

        at = strstr(strid,"@");
        *at = '%';
        id = jid_new(id->p,i->me);
        jid_set(id,strid,JID_USER);
    }

    log_debug(ZONE,"2mx returning %s",jid_full(id));
    return id;
}


void smtp(instance i, xmlnode x)
{
    xmlnode config, cur;
    xdbcache xc;
    smtpi si;

    log_debug(ZONE,"smtp loading");

    xc = xdb_cache(i);
    config = xdb_get(xc, jid_new(xmlnode_pool(x),"config@-internal"),"jabber:config:smtp");

    /* set up internal instance and hash tables */
    si = pmalloco(i->p,sizeof(_smtpi));
    si->i = i;
    si->config = xmlnode_dup_pool(i->p, config);
    si->timeout = j_atoi(xmlnode_get_tag_data(config,"timeout"),30);
    if((si->me = xmlnode_get_tag_data(si->config,"mxhost")) == NULL)
    {
        log_error("smtp","SMTP Server must have a default SMTP <mxhost>smtp.host.foo</mxhost> configured");
        return;
    }
    for(cur = xmlnode_get_tag(config,"open");cur != NULL; xmlnode_hide(cur), cur = xmlnode_get_tag(config,"open"))
        if(xmlnode_get_data(cur) == NULL)
            si->open = 1;

    if((si->relay = xmlnode_get_tag_data(si->config,"relay")) == NULL)
        si->relay = pstrdup(i->p, "localhost");

    register_phandler(i, o_DELIVER, smtp_out_packets, (void *)si);

    if((cur = xmlnode_get_tag(config,"ip")) != NULL)
        for(;cur != NULL; xmlnode_hide(cur), cur = xmlnode_get_tag(config,"ip"))
        {
            mio_listen(j_atoi(xmlnode_get_attrib(cur, "port"), 5225), xmlnode_get_data(cur), smtp_in_read, (void*)si, NULL, NULL);
        }
    else /* no special config, use defaults */
        mio_listen(5225, NULL, smtp_in_read, (void *)si, NULL, NULL);


    si->spamcheck = xmlnode_get_tag_data(si->config,"spamcheck");
    xmlnode_free(config);
}

smtps smtp_new(mio m)
{
    smtps s;
    s = pmalloco(mio_pool(m), sizeof(_smtps));
    s->m = m;
    s->p = mio_pool(m);
    s->message = spool_new(s->p);

    return s;
}
